﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class MainMenu : MonoBehaviour 
{
	private AudioSource title;
	private AudioSource battle;
	private AudioSource cursorFX;
	private AudioSource selectFX;

	public Image cursorStart; 
	public Image cursorExit; 

	public GameObject pressAnyButton; 
	public GameObject start;
	public GameObject exit; 

	public string acceptButton = "Accept_P1";
	public string dPadAxis = "D-Pad_P1"; 
	public string moveVerticalAxis = "Vertical_P1";

	private bool playOnce = true;
	private bool pressAnyButtonOnce = true;

	void Start()
	{
		Cursor.visible = false;

		pressAnyButton.SetActive(true);
		start.SetActive(false);
		exit.SetActive(false);

		cursorExit.enabled = false;
		cursorStart.enabled = false;

		title = GameObject.Find ("Title Music").GetComponent<AudioSource> ();
		battle = GameObject.Find ("Battle").GetComponent<AudioSource> ();
		selectFX = GameObject.Find ("Menu Select").GetComponent<AudioSource> ();
		cursorFX = GameObject.Find ("Menu Cursor").GetComponent<AudioSource> ();
	}

	void Update()
	{
		if (Input.GetKeyDown(KeyCode.Joystick1Button7) || Input.GetKeyDown(KeyCode.Return) && pressAnyButtonOnce) 				
			StartCoroutine(PressAnyButton());		

		if (Input.GetAxisRaw (dPadAxis) == 0 && Input.GetAxisRaw (moveVerticalAxis) == 0)
			playOnce = true;

		if ((Input.GetAxisRaw (dPadAxis) < 0 || Input.GetAxisRaw (moveVerticalAxis) < 0) && !pressAnyButtonOnce) 
		{
			cursorStart.enabled = true;
			cursorExit.enabled = false;
			if (playOnce) 
			{
				cursorFX.Play ();
				playOnce = false;
			}
		} 

		if((Input.GetAxisRaw(dPadAxis) > 0 || Input.GetAxisRaw (moveVerticalAxis) > 0) && !pressAnyButtonOnce)
		{
			cursorStart.enabled = false;
			cursorExit.enabled = true;
			if (playOnce) 
			{
				cursorFX.Play ();
				playOnce = false;
			}
		}

		if (cursorStart.enabled && Input.GetButtonDown (acceptButton))
		{
			selectFX.Play ();
			battle.Play ();
			title.Stop ();
			SceneManager.LoadScene ("Game");
		}			
		if (cursorExit.enabled && Input.GetButtonDown (acceptButton)) 
		{
			selectFX.Play ();
			Application.Quit ();
		}			
	}

	IEnumerator PressAnyButton()
	{
		selectFX.Play ();
		pressAnyButtonOnce = false;
		pressAnyButton.SetActive (false);
		start.SetActive (true);
		exit.SetActive (true);
		yield return new WaitForSeconds (0.1f);
		cursorStart.enabled = true;
	}
}
